@@ -1,12 +1,11 @@
 
 """Tasks are the main abstractions managed by doit"""
 
-import types
 import os
 import sys
 import inspect
 from collections import OrderedDict
-from functools import partial
+from collections.abc import Callable
 from pathlib import PurePath
 
 from .cmdparse import CmdOption, TaskParse
@@ -153,9 +152,9 @@ class Task(object):
                   'pos_arg': (string_types, (None,)),
                   'verbosity': ((), (None, 0, 1, 2,)),
                   'getargs': ((dict,), ()),
-                  'title': ((types.FunctionType,), (None,)),
+                  'title': ((Callable,), (None,)),
                   'watch': ((list, tuple), ()),
-    }
+                  }
 
 
     def __init__(self, name, actions, file_dep=(), targets=(),
@@ -231,6 +230,7 @@ class Task(object):
         self.values = {}
         self.verbosity = verbosity
         self.custom_title = title
+        self.cfg_values = None
 
         # clean
         if clean is True:
@@ -359,17 +359,28 @@ class Task(object):
             self._expand_map[dep](self, dep_values)
 
 
-    def init_options(self):
+    def init_options(self, args=None):
         """Put default values on options.
 
-        This will only be used, if params options were not passed
-        on the command line.
+        This function will only initialize task options once. If provided the args
+        parameter will be parsed for command line arguments intended for this task.
+
+        Return value: unparsed command line task arguments or None.
         """
         if self.options is None:
+            self.options = {}
             taskcmd = TaskParse([CmdOption(opt) for opt in self.params])
-            # ignore positional parameters
-            self.options = taskcmd.parse('')[0]
+            if self.cfg_values is not None:
+                taskcmd.overwrite_defaults(self.cfg_values)
 
+            if args is None:
+                # ignore positional parameters
+                self.options.update(taskcmd.parse('')[0])
+                return None
+            else:
+                parsed_options, args = taskcmd.parse(args)
+                self.options.update(parsed_options)
+                return args
 
     def _init_getargs(self):
         """task getargs attribute define implicit task dependencies"""
@@ -408,7 +419,7 @@ class Task(object):
         @param valid (list): of valid types/value accepted
         @raises InvalidTask if invalid input
         """
-        if type(value) in valid[0]:
+        if isinstance(value, valid[0]):
             return
         if value in valid[1]:
             return
@@ -574,31 +585,22 @@ def dict_to_task(task_dict):
 
 def clean_targets(task, dryrun):
     """remove all targets from a task"""
-    files = [path for path in task.targets if os.path.isfile(path)]
-    dirs = [path for path in task.targets if os.path.isdir(path)]
-
-    # remove all files
-    for file_ in files:
-        print("%s - removing file '%s'" % (task.name, file_))
-        if not dryrun:
-            os.remove(file_)
-
-    # remove all directories (if empty)
-    for dir_ in dirs:
-        if os.listdir(dir_):
-            msg = "%s - cannot remove (it is not empty) '%s'"
-            print(msg % (task.name, dir_))
-        else:
-            msg = "%s - removing dir '%s'"
-            print(msg % (task.name, dir_))
+    for target in sorted(task.targets, reverse=True):
+        if os.path.isfile(target):
+            print("%s - removing file '%s'" % (task.name, target))
             if not dryrun:
-                os.rmdir(dir_)
+                os.remove(target)
+        elif os.path.isdir(target):
+            if os.listdir(target):
+                msg = "%s - cannot remove (it is not empty) '%s'"
+                print(msg % (task.name, target))
+            else:
+                msg = "%s - removing dir '%s'"
+                print(msg % (task.name, target))
+                if not dryrun:
+                    os.rmdir(target)
 
 
-def _return_param(val):
-    '''just return passed parameter - make a callable from any value'''
-    return val
-
 # uptodate
 class result_dep(UptodateCalculator):
     """check if result of the given task was modified
@@ -635,15 +637,23 @@ class result_dep(UptodateCalculator):
                 sub_tasks[sub] = self.get_val(sub, 'result:')
         return sub_tasks
 
-    def __call__(self, task, values):
-        """return True if result is the same as last run"""
-        dep_task = self.tasks_dict[self.dep_name]
+    def _get_dep_result(self, dep_task):
         if not dep_task.has_subtask:
             dep_result = self._result_single()
         else:
             dep_result = self._result_group(dep_task)
-        func = partial(_return_param, {self.result_name: dep_result})
-        task.value_savers.append(func)
+        return dep_result
+
+
+    def __call__(self, task, values):
+        """return True if result is the same as last run"""
+        dep_task = self.tasks_dict[self.dep_name]
+        dep_result = self._get_dep_result(dep_task)
+
+        def result_saver():
+            # get latest value after execution of dependent task
+            return {self.result_name: self._get_dep_result(dep_task)}
+        task.value_savers.append(result_saver)
 
         last_success = values.get(self.result_name)
         if last_success is None:
